#+TITLE: Sacha Chua's Emacs configuration
#+ELEVENTY_COLLECTIONS: _posts
#+ELEVENTY_BASE_DIR: ~/proj/static-blog/
#+ELEVENTY_CATEGORIES+: emacs
#+ELEVENTY_LAYOUT: layouts/post
#+ELEVENTY_BASE_URL: https://sachachua.com
* STARTED Finding my blog posts with consult-omni                     :emacs:
:PROPERTIES:
:CUSTOM_ID: completion-consult-consult-omni-blog-posts
:Effort:   0:30
:EXPORT_DATE: 2025-07-21T23:12:28-0400
:EXPORT_ELEVENTY_PERMALINK: /blog/2025/07/finding-my-blog-posts-with-consult-omni/
:EXPORT_ELEVENTY_FILE_NAME: blog/2025/07/finding-my-blog-posts-with-consult-omni/
:EXPORT_MASTODON: https://social.sachachua.com/@sacha/statuses/01K0R38KMPAKMCTEBHM2VS69TX
:END:
:LOGBOOK:
CLOCK: [2025-07-21 Mon 22:27]--[2025-07-21 Mon 23:16] =>  0:49
:END:

Sometimes I just want to quickly get to a blog post by title. I use [[https://sachachua.com/blog/2024/10/yay-emacs-inserting-links-with-consult-omni/][consult-omni]] for quick web searches that I can jump to or insert as a link. Sure, I can limit this search to my blog by specifying site:sachachua.com or using the code I wrote to [[dotemacs:searching-my-blog][search my blog, notes, and sketches with consult-ripgrep and consult-omni]], but the search sometimes gets confused by other text on the page. When I [[https://sachachua.com/blog/category/11ty/][publish my blog with Eleventy]], I also create a JSON file with [[https://sachachua.com/blog/all/index.json][all my blog post URLs and titles]]. Here's how I can use that data as a consult-omni source.

#+begin_src emacs-lisp
(defun my-consult-omni-blog-data ()
	(let ((base (replace-regexp-in-string "/$" "" my-blog-base-url))
				(json-object-type 'alist)
				(json-array-type 'list))
		(mapcar
		 (lambda (o)
			 (list :url (concat base (alist-get 'permalink o))
						 :title (alist-get 'title o)
						 :date (alist-get 'date o)))
		 (sort (json-read-file "~/sync/static-blog/_site/blog/all/index.json")
					 (lambda (a b)
						 (string< (or (alist-get 'date b) "")
											(or (alist-get 'date a) "")))))))
(unless (get 'my-consult-omni-blog-data :memoize-original-function)
	(memoize #'my-consult-omni-blog-data "5 minutes"))

(defun my-consult-omni-blog-titles-builder (input &rest args &key callback &allow-other-keys)
	(let* ((quoted (when input (regexp-quote input)))
				 (list
					(if quoted
							(seq-filter
							 (lambda (o)
								 ;; TODO: Someday figure out orderless?
								 (string-match quoted (concat (plist-get o :title) " - " (plist-get o :title))))
							 (my-consult-omni-blog-data))
						(my-consult-omni-blog-data)))
				 (candidates
					(mapcar
					 (lambda (o)
						 (propertize
							(concat (plist-get o :title))
							:source "Blog"
							:date (plist-get o :date)
							:title (plist-get o :title)
							:url (plist-get o :url)))
					 (if quoted (seq-take list 3) list))))
		(when callback (funcall callback candidates))
		candidates))

(defun my-consult-omni-blog-annotation (s)
	(format " (%s)"
					(propertize (substring (or (get-text-property 0 :date s) "") 0 4)
											'face 'completions-annotations)))

(with-eval-after-load 'consult-omni
  (consult-omni-define-source
	 "My blog"
	 :narrow-char ?b
   :type 'sync
	 :request #'my-consult-omni-blog-titles-builder
	 :on-return 'my-consult-org-bookmark-visit
   :group #'consult-omni--group-function
	 :annotate #'my-consult-omni-blog-annotation
   :min-input 3
	 :sort nil
   :require-match t))
#+end_src

Here's what it looks like by itself when I call ~consult-omni-my-blog~:

#+CAPTION: Screenshot of a search through my blog post titles
[[file:images/2025-07-21_23-24-09.png]]

Then I can add it as one of the sources used by ~consult-omni~:

#+begin_src emacs-lisp :tangle no :eval no
	(setq consult-omni-multi-sources
				'(consult-omni--source-google
					consult-omni--source-my-org-bookmarks
					consult-omni--source-my-blog))
#+end_src

Here's what it looks like when I call ~consult-omni~ to search my bookmarks, blog posts, and Google results at the same time.

#+CAPTION: Screenshot of a consult-omni search with multiple sources
[[file:images/2025-07-21_23-29-06.png]]

Then I can press ~RET~ to open the blog post in my browser or ~C-. i l~ to [[dotemacs:consult-omni][insert the link using Embark]] (~my-consult-omni-embark-insert-link~).

Related:
- [[dotemacs:completion-consult-consult-omni-bookmarks][consult-omni--source-my-org-bookmarks]]
- [[dotemacs:completing-blog-posts][Completing blog posts]] (Org link type with completion)

_Next steps:_ I want to find out how to get ~:sort nil~ to be respected so that more recent blog posts are listed first. Also, it might be fun to define a similar source for the sections of my Emacs configuration, like the way I can use my [[dotemacs:links-to-my-config][dotemacs: Org Mode link type with completion]].

